
library tags.html;

import 'dart:html';

// -----------------------------------------------------------------------------
// Html Tags API.
// -----------------------------------------------------------------------------

a         ([void build()]) => _build(new AnchorElement(), build);
area      ([void build()]) => _build(new AreaElement(), build);
base      ([void build()]) => _build(new BaseElement(), build);
body      ([void build()]) => _build(new BodyElement(), build);
br        ([void build()]) => _build(new BRElement(), build);
button    ([void build()]) => _build(new ButtonElement(), build);
canvas    ([void build()]) => _build(new CanvasElement(), build);
caption   ([void build()]) => _build(new TableCaptionElement(), build);
col       ([void build()]) => _build(new TableColElement(), build);
content   ([void build()]) => _build(new ContentElement(), build);
datalist  ([void build()]) => _build(new DataListElement(), build);
details   ([void build()]) => _build(new DetailsElement(), build);
div       ([void build()]) => _build(new DivElement(), build);
embed     ([void build()]) => _build(new EmbedElement(), build);
fieldset  ([void build()]) => _build(new FieldSetElement(), build);
form      ([void build()]) => _build(new FormElement(), build);
h1        ([void build()]) => _build(new HeadingElement.h1(), build);
h2        ([void build()]) => _build(new HeadingElement.h2(), build);
h3        ([void build()]) => _build(new HeadingElement.h3(), build);
h4        ([void build()]) => _build(new HeadingElement.h4(), build);
h5        ([void build()]) => _build(new HeadingElement.h5(), build);
h6        ([void build()]) => _build(new HeadingElement.h6(), build);
head      ([void build()]) => _build(new HeadElement(), build);
hr        ([void build()]) => _build(new HRElement(), build);
html      ([void build()]) => _build(new HtmlElement(), build);
img       ([void build()]) => _build(new ImageElement(), build);
input     ([void build()]) => _build(new InputElement(), build);
keygen    ([void build()]) => _build(new KeygenElement(), build);
label     ([void build()]) => _build(new LabelElement(), build);
legend    ([void build()]) => _build(new LegendElement(), build);
li        ([void build()]) => _build(new LIElement(), build);
link      ([void build()]) => _build(new LinkElement(), build);
map       ([void build()]) => _build(new MapElement(), build);
menu      ([void build()]) => _build(new MenuElement(), build);
meter     ([void build()]) => _build(new MeterElement(), build);
object    ([void build()]) => _build(new ObjectElement(), build);
ol        ([void build()]) => _build(new OListElement(), build);
optgroup  ([void build()]) => _build(new OptGroupElement(), build);
option    ([void build()]) => _build(new OptionElement(), build);
output    ([void build()]) => _build(new OutputElement(), build);
p         ([void build()]) => _build(new ParagraphElement(), build);
param     ([void build()]) => _build(new ParamElement(), build);
pre       ([void build()]) => _build(new PreElement(), build);
progress  ([void build()]) => _build(new ProgressElement(), build);
script    ([void build()]) => _build(new ScriptElement(), build);
select    ([void build()]) => _build(new SelectElement(), build);
source    ([void build()]) => _build(new SourceElement(), build);
span      ([void build()]) => _build(new SpanElement(), build);
style     ([void build()]) => _build(new StyleElement(), build);
table     ([void build()]) => _build(new TableElement(), build);
tbody     ([void build()]) => _build(_element('tbody'), build);
td        ([void build()]) => _build(new TableCellElement(), build);
textarea  ([void build()]) => _build(new TextAreaElement(), build);
tfoot     ([void build()]) => _build(_element('tfoot'), build);
th        ([void build()]) => _build(_element('th'), build);
thead     ([void build()]) => _build(_element('thead'), build);
title     ([void build()]) => _build(new TitleElement(), build);
tr        ([void build()]) => _build(new TableRowElement(), build);
track     ([void build()]) => _build(new TrackElement(), build);
ul        ([void build()]) => _build(new UListElement(), build);
video     ([void build()]) => _build(new VideoElement(), build);

// -----------------------------------------------------------------------------
// Implementation.
// -----------------------------------------------------------------------------

List<Element> _elements = new List();

Element _build(Element e, void build()) {
  if (!_elements.isEmpty) _elements.last.append(e);  
  if (build == null) return e;
  _elements.add(e);  
  try {
    build();
  } finally {
    _elements.removeLast();
  }
  return e;
}

// Create an Element for a `tag` that is untyped in `dart:html`.  E.g. `<th>`
_element(String tag) {
  if (!Element.isTagSupported(tag)) throw new UnsupportedError(tag);
  return new Element.tag(tag);
}
